import csv
from typing import Optional, Iterable

import numpy as np
import pandas as pd


def save_np_as_horizontal_csv(
    dest: str, arr: np.typing.NDArray, columns: Optional[Iterable] = None
):
    r"""
    Saves a numpy array as a horizontal csv of the shape rows x columns where df[row][col] is a list of features
    """
    arr = arr.tolist()
    with open(dest, "w", newline="") as f:
        w = csv.writer(f, delimiter=",")
        if columns:
            w.writerow(columns)
        w.writerows(arr)


def save_np_as_vertical_csv(
    dest: str, arr: np.typing.NDArray, regions: list[str], features: list[str]
):
    r"""
    Saves a numpy array as a vertical csv of the shape (timestamps * rows) x columns
    """
    arr = arr.tolist()
    with open(dest, "w", newline="") as f:
        w = csv.writer(f, delimiter=",")
        w.writerow(["time", "region"] + features)
        for t, r in enumerate(arr):
            for i, d in enumerate(r):
                w.writerow([t] + [regions[i]] + d)


def load_horizontal_csv(dest: str):
    r"""
    Loads a horizontal csv generated by mlossp into a numpy array.
    """
    df = pd.read_csv(dest).to_numpy()
    return np.array(
        [[z[1:-1].replace("'", "").split(", ") for z in x] for x in df], dtype=float
    )
